'use strict';

var preact = require('preact');

function toVnode(components, node, originalHtml) {
  // get element name or component name if registered
  const name = components[node.name] || node.name;
  // convert props to strings
  const props = {};
  for (let k in node.attrs) {
    props[k] = node.attrs[k].map(n => n.content).join('');
  }
  // if there is a compressed original source, add it as _state prop
  if (originalHtml) {
    props._state = originalHtml;
  }
  // content is either a string, a subtree, or there isn't any
  if (typeof node.content === 'string') {
    return preact.h(name, props, node.content)
  } else if (Array.isArray(node.content)) {
    const subtree = node.content.map(n => {
      if (n.type === 'tag') return toVnode(components, n)
      if (n.type === 'text') return n.content
    });
    return preact.h(name, props, subtree)
  } else {
    return preact.h(name, props)
  }
}

module.exports = toVnode;
